"""Scatter plots of various examples where Pearson correlation fails."""

from ennemi import estimate_corr
import matplotlib.pyplot as plt
from matplotlib.gridspec import GridSpec
import numpy as np

rng = np.random.default_rng(2020_06_01)
N = 120

# A sine curve: y = sin(x) + eps
x1_raw = rng.uniform(-5, 5, N)
x1_sin = np.sin(x1_raw)
y1 = x1_sin + rng.normal(0.0, 0.2, N)

# A symmetric absolute value function
x2_raw = rng.uniform(-1, 1, N)
x2_abs = np.abs(x2_raw)
abs_errors = rng.normal(0.0, 0.2, N)
y2 = x2_abs + abs_errors

# An asymmetric absolute value function
x3_raw = x2_raw + 0.4
x3_abs = np.abs(x3_raw)
y3 = x3_abs + abs_errors

# Draw the plot
plt.rc("font", size=8)
plt.rc("figure", dpi=300)

fig = plt.figure(figsize=(8, 3))
gs = GridSpec(2, 3, height_ratios=[8, 1])

def plot_scatter(x, y, ax, title, number):
    a, b = np.polyfit(x, y, deg=1)

    ax.scatter(x, y, marker=".", color="k")
    ax.plot(x, a*x + b, color="C0")
    ax.tick_params(labelbottom=False, labelleft=False, bottom=False, left=False)
    ax.set_title(title)
    ax.set_title(f"({number})", loc="left")

plot_scatter(x1_raw, y1, fig.add_subplot(gs[0,0]), r"$y = \sin(x) + \varepsilon$", "a")
plot_scatter(x2_raw, y2, fig.add_subplot(gs[0,1]), r"$y = |x| + \varepsilon$", "b")
plot_scatter(x3_raw, y3, fig.add_subplot(gs[0,2]), r"$y = |x + 0.4| + \varepsilon$", "c")

def plot_stats(x_raw, x_trans, y, ax):
    corr_raw = np.corrcoef(x_raw, y)[0,1]
    corr_trans = np.corrcoef(x_trans, y)[0,1]
    corr_mi = estimate_corr(x_raw, y)[0,0]

    text = f"Original: {corr_raw:.2f}\nTransformed: {corr_trans:.2f}\nMI: {corr_mi:.2f}"

    ax.set_frame_on(False)
    ax.set_axis_off()
    ax.text(0, 1, text, ha="left", va="top", transform=ax.transAxes)

plot_stats(x1_raw, x1_sin, y1, fig.add_subplot(gs[1,0]))
plot_stats(x2_raw, x2_abs, y2, fig.add_subplot(gs[1,1]))
plot_stats(x3_raw, x3_abs, y3, fig.add_subplot(gs[1,2]))


plt.savefig("correlation_examples.pdf", transparent=True)

