"""Precision of MI correlation as a function of sample size and logarithm transform.

This script is quite slow because it does not implement any parallelization.
Built-in parallelization of ennemi cannot be used because the estimates are
required to be completely independent."""

from ennemi import estimate_corr
import matplotlib.pyplot as plt
from matplotlib.gridspec import GridSpec
import numpy as np

SAMPLES = 20
ITERATIONS = 100

rng = np.random.default_rng(2020_06_01)
mi_raw = np.empty((ITERATIONS, SAMPLES))
mi_norm = np.empty((ITERATIONS, SAMPLES))
sample_sizes = np.logspace(np.log10(50), 4, SAMPLES, dtype=np.int32)

for i in range(ITERATIONS):
    if i % 10 == 0:
        print(f"Iteration {i+1}/{ITERATIONS}")

    cov1 = np.asarray([[1, 0.6], [0.6, 1]])
    norm_data = rng.multivariate_normal([0, 0], cov1, 10**4)
    raw_data = norm_data.copy()
    raw_data[:,0] = np.exp(raw_data[:,0])
    raw_data[:,1] = np.exp(2 * raw_data[:,1])

    for (j, ss) in enumerate(sample_sizes):
        mi_raw[i, j] = estimate_corr(raw_data[:ss,0], raw_data[:ss,1])
        mi_norm[i, j] = estimate_corr(norm_data[:ss,0], norm_data[:ss,1])


# Draw the plot
plt.rc("font", size=8)
plt.rc("figure", dpi=300)
plt.rc("xtick", labelsize="small")
plt.rc("ytick", labelsize="small")

fig = plt.figure(figsize=(8, 4))
gs = GridSpec(2, 2, width_ratios=[5, 3])

ax = fig.add_subplot(gs[0:,0])
ax.axhline(0.6, linestyle="--", color="k")

ax.plot(sample_sizes, np.quantile(mi_norm, 0.5, axis=0),
    label="Normal: $X \\sim Y$", color="C3")
ax.fill_between(sample_sizes,
    np.quantile(mi_norm, 0.25, axis=0),
    np.quantile(mi_norm, 0.75, axis=0), color="C3", alpha=0.2)
ax.plot(sample_sizes, np.quantile(mi_raw, 0.5, axis=0),
    label="Log-normal: $\\exp(X) \\sim \\exp(2Y)$", color="C4")
ax.fill_between(sample_sizes,
    np.quantile(mi_raw, 0.25, axis=0),
    np.quantile(mi_raw, 0.75, axis=0), color="C4", alpha=0.2)

ax.legend(loc="lower right")
ax.set_xscale("log")
ax.set_xlabel("Sample size")
ax.set_ylabel("Correlation coefficient")
ax.set_title("(a)", loc="left")
ax.grid()


# Draw example scatter plots of the lognormal and normal data sets
norm_data = rng.multivariate_normal([0, 0], cov1, 100)

ax = fig.add_subplot(gs[0,1])
ax.scatter(norm_data[:,0], norm_data[:,1],
    marker=".", color="C3", alpha=0.3)
ax.set_title("(b)", loc="left")
ax.set_xticks([-2, -1, 0, 1, 2])
ax.set_xticklabels(["", "-1", "0", "1", "2"]) # Overlap with "(c)" label

ax = fig.add_subplot(gs[1,1])
ax.scatter(np.exp(norm_data[:,0]), np.exp(2 * norm_data[:,1]),
    marker=".", color="C4", alpha=0.3)
ax.set_title("(c)", loc="left")

plt.savefig("lognormal_ci.pdf", transparent=True)

