"""Plots of time lagged correlation with 3-25nm particle count."""

import matplotlib.pyplot as plt
from matplotlib import colors
from matplotlib.gridspec import GridSpec
import numpy as np
import pandas as pd

# Import the analysis results
results_uncond = pd.read_csv("lag-CS-all-RESULTS.csv", index_col=0)
results_doy_hour = pd.read_csv("lag-CS-all-doy-hour-RESULTS.csv", index_col=0)

# Reorder the final subset of columns
columns = [
    "J 3-25 nm", "3-25 nm", "25-100 nm", "100-1000 nm",
    "T42", "T336", "RHTd",
    "WSU336", "WD", "Glob",
    "CO", "O3", "NOx", "SO2",
    "H2SO4_prox",# "CS",
    "Hour", "DayOfYear"
]
column_names = [
    "3-25 nm form. rate", "3-25 nm conc.", "25-100 nm conc.", "100-1000 nm conc.",
    "Temperature (4 m)", "Temperature (34 m)", "Relative humidity",
    "Wind speed (34 m)", "Wind direction", "Global rad.",
    "$\\mathrm{CO}$", "$\\mathrm{O_3}$", "$\\mathrm{NOx}$", "$\\mathrm{SO_2}$",
    "$\\mathrm{H_2 SO_4}$ proxy",# "Condensation sink",
    "Hour", "Day of year"
]
results_uncond = results_uncond[columns]
results_doy_hour = results_doy_hour[columns]

# Draw the plot
plt.rc("font", size=8)
plt.rc("figure", dpi=300)

# Custom discretized colorbar, same as in fig_pairwise
vir = plt.get_cmap("viridis")
color_list = vir([0, 0.25, 0.48, 0.61, 0.74, 0.87, 1.0])
boundaries = [0.15, 0.25, 0.35, 0.45, 0.55, 0.65, 0.75, 0.85]
cmap = colors.ListedColormap(color_list)
norm = colors.BoundaryNorm(boundaries, len(color_list), clip=True)

def plot_slice(results, ax, title, number, rightmost=False):
    ax.set_facecolor("white")
    mesh = ax.pcolormesh(results.T, cmap=cmap, norm=norm)
    if rightmost:
        fig.colorbar(mesh, label="MI correlation coefficient", extend="both",
            ticks=[0.25, 0.35, 0.45, 0.55, 0.65, 0.75])

    # Center the tick marks on each lag, and invert the axis for natural reading
    ax.set_xticks(np.arange(0, 24+1, 2) + 0.5)
    ax.set_xticklabels(np.arange(0, 24+1, 2))
    ax.invert_xaxis()
    ax.set_xlabel("Covariate lag (hours)")

    ax.set_yticks(np.arange(len(columns)) + 0.5)
    if not rightmost:
        ax.set_yticklabels(column_names)
    else:
        ax.set_yticklabels([""] * len(columns))
    ax.invert_yaxis()

    ax.set_title(title)
    ax.set_title(f"({number})", loc="left")

fig = plt.figure(figsize=(8, 3.5))
gs = GridSpec(1, 2, width_ratios=[16, 20])
plot_slice(results_uncond, fig.add_subplot(gs[0,0]), "Unconditional MI with CS", "a")
plot_slice(results_doy_hour, fig.add_subplot(gs[0,1]), "MI given day of year and hour", "b", True)

plt.tight_layout()
plt.savefig("fig_lagged.pdf", transparent=True)

