"""Matrix plots of pairwise correlation."""

import matplotlib.pyplot as plt
from matplotlib import colors
from matplotlib.gridspec import GridSpec
import numpy as np
import pandas as pd

# Import the analysis results
def import_results(filename):
    results = pd.read_csv(filename, index_col=0)
    results = results.rename(dict(enumerate(results.columns)), axis="index")
    return results

results_uncond = import_results("pairwise-5%-none-RESULTS.csv")
results_doy_only = import_results("pairwise-5%-doy-RESULTS.csv")
results_doy_hour = import_results("pairwise-5%-doy-hour-RESULTS.csv")

# Reorder the final subset of columns
columns = [
    "J 3-25 nm", "3-25 nm", "25-100 nm", "100-1000 nm",
    "T42", "T336", "RHTd",
    "WSU336", "WD", "Glob",
    "CO", "O3", "NOx", "SO2",
    "H2SO4_prox", "CS",
    "Hour", "DayOfYear"
]
column_names = [
    "3-25 nm form. rate", "3-25 nm conc.", "25-100 nm conc.", "100-1000 nm conc.",
    "Temperature (4 m)", "Temperature (34 m)", "Relative humidity",
    "Wind speed (34 m)", "Wind direction", "Global rad.",
    "$\\mathrm{CO}$", "$\\mathrm{O_3}$", "$\\mathrm{NOx}$", "$\\mathrm{SO_2}$",
    "$\\mathrm{H_2 SO_4}$ proxy", "Condensation sink",
    "Hour", "Day of year"
]
short_column_names = [
    "3-25 nm FR", "3-25 nm", "25-100 nm", "100-1000 nm",
    "Temp 4 m", "Temp 34 m", "RH",
    "Wind sp.", "Wind dir", "Global rad.",
    "$\\mathrm{CO}$", "$\\mathrm{O_3}$", "$\\mathrm{NO_x}$", "$\\mathrm{SO_2}$",
    "$\\mathrm{H_2 SO_4}$ proxy", "CS",
    "Hour", "Day of year"
]
results_uncond = results_uncond.loc[columns, columns]
results_doy_only = results_doy_only.loc[columns, columns]
results_doy_hour = results_doy_hour.loc[columns, columns]



# Draw the plot
plt.rc("font", size=8)
plt.rc("figure", dpi=300)

# Custom discretized colorbar for improved distiction between colors
# The first two colors are darker, the rest are uniform
vir = plt.get_cmap("viridis")
color_list = vir([0, 0.25, 0.48, 0.61, 0.74, 0.87, 1.0])
boundaries = [0.15, 0.25, 0.35, 0.45, 0.55, 0.65, 0.75, 0.85]
cmap = colors.ListedColormap(color_list)
norm = colors.BoundaryNorm(boundaries, len(color_list), clip=True)

def plot_results(results, ax, title, tag, rightmost=True, bottommost=False):
    ax.set_facecolor("white")
    mesh = ax.pcolormesh(results, cmap=cmap, norm=norm)
    if rightmost:
        fig.colorbar(mesh, label="MI correlation coefficient", extend="both",
            ticks=[0.25, 0.35, 0.45, 0.55, 0.65, 0.75])

    ax.set_xticks(np.arange(len(columns)) + 0.5)
    if not bottommost:
        ax.set_xticklabels(short_column_names)
    else:
        ax.set_xticklabels([""] * len(columns))
    ax.tick_params(axis="x", labelrotation=90.0)

    ax.set_yticks(np.arange(len(columns)) + 0.5)
    if not rightmost:
        ax.set_yticklabels(column_names)
    else:
        ax.set_yticklabels([""] * len(columns))
    ax.invert_yaxis()

    ax.set_title(title)
    ax.set_title(tag, loc="left")

fig = plt.figure(figsize=(8, 7))
gs = GridSpec(2, 3, width_ratios=[16, 1, 20])
plot_results(results_uncond, fig.add_subplot(gs[0,0]), "Unconditional MI", "(a)",
    rightmost=False)
plot_results(results_doy_only, fig.add_subplot(gs[0,2]), "MI given day of year", "(b)")
plot_results(results_doy_hour, fig.add_subplot(gs[1,0]), "MI given day of year and hour", "(c)",
    bottommost=True, rightmost=False)


# Difference matrix
ax = fig.add_subplot(gs[1,2])
ax.set_facecolor("white")
mesh = ax.pcolormesh(results_doy_hour - results_uncond,
    cmap=plt.get_cmap("coolwarm"), vmin=-0.5, vmax=0.5)
fig.colorbar(mesh, label="MI correlation difference", extend="both")

ax.set_xticks(np.arange(len(columns)) + 0.5)
ax.set_xticklabels([""] * len(columns))
ax.tick_params(axis="x", labelrotation=90.0)

ax.set_yticks(np.arange(len(columns)) + 0.5)
ax.set_yticklabels([""] * len(columns))
ax.invert_yaxis()

ax.set_title("Difference on conditioning")
ax.set_title("(d)", loc="left")



plt.tight_layout()
plt.savefig("fig_pairwise.pdf", transparent=True)
