"""Scatter plots between some variables, partitioned seasonally."""

from matplotlib.gridspec import GridSpec
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from scipy.optimize import curve_fit

# Use the data at any time of day, but separated by season
# Select a small random subset for plotting
data = pd.read_hdf("smear-data.hdf5", "raw_data")

rng = np.random.default_rng(2022_03_30)
frac = 1000 / data.shape[0]
mask = rng.choice([False, True], data.shape[0], replace=True, p=[1-frac, frac])
data = data[mask]

summer = (data.index.month == 6) | (data.index.month == 7) | (data.index.month == 8)
winter = (data.index.month == 12) | (data.index.month == 1) | (data.index.month == 2)


# Draw the plot
plt.rc("font", size=8)
plt.rc("figure", dpi=300)
plt.rc("xtick", labelsize="small")
plt.rc("ytick", labelsize="small")

fig = plt.figure(figsize=(8, 4))
gs = GridSpec(2, 3, height_ratios=[8, 1])


def plot_loglog_fit_line(ax, x, y, col):
    not_na = x.notna() & y.notna() & (x > 1e-9) & (y > 1e-9)
    x = np.log(x[not_na])
    y = np.log(y[not_na])

    # Fit a linear model on the log-transformed variables
    a, b = np.polyfit(x, y, deg=1)
    ran = np.asarray([np.min(x), np.max(x)])
    ax.plot(np.exp(ran), np.exp(a*ran + b), "-", color=col)

    # Also return the fitted correlation
    return np.corrcoef(a*x+b, y)[0,1]

def plot_logy_fit_line(ax, x, y, col):
    not_na = x.notna() & y.notna() & (y > 1e-9)
    x = x[not_na]
    y = np.log(y[not_na])

    # Linear model where y is log-transformed
    a, b = np.polyfit(x, y, deg=1)
    ran = np.asarray([np.min(x), np.max(x)])
    ax.plot(ran, np.exp(a*ran + b), "-", color=col)

    return np.corrcoef(a*x+b, y)[0,1]

def plot_triglog_fit_line(ax, x, y, col):
    not_na = x.notna() & y.notna() & (y > 1e-9)
    x = x[not_na]
    y = np.log(y[not_na])

    # Fit a two-level trigonometric model
    CIRCLE = 2 * np.pi/360
    HALF_CIRCLE = 2 * CIRCLE

    def model(x, a1, a2, b1, b2, c):
        return a1*np.sin(x*CIRCLE) + a2*np.cos(x*CIRCLE) + \
            b1*np.sin(x*HALF_CIRCLE) + b2*np.cos(x*HALF_CIRCLE) + c
    popt, _ = curve_fit(model, x, y)

    ran = np.linspace(np.min(x), np.max(x), 100)
    ax.plot(ran, np.exp(model(ran, *popt)), "-", color=col)

    return np.corrcoef(model(x, *popt), y)[0,1]


# NOx and CS
ax = fig.add_subplot(gs[0,0])

corr = plot_loglog_fit_line(ax, data.loc[summer, "NOx"], data.loc[summer, "CS"], "C1")
ax.scatter(data.loc[summer, "NOx"], data.loc[summer, "CS"],
    color="C1", marker=",", s=3, label=f"Jun-Aug ($|\\rho|={{{corr:.2}}}$)")

corr = plot_loglog_fit_line(ax, data.loc[winter, "NOx"], data.loc[winter, "CS"], "C0")
ax.scatter(data.loc[winter, "NOx"], data.loc[winter, "CS"],
    color="C0", marker=".", s=9, label=f"Dec-Feb ($|\\rho|={{{corr:.2}}}$)")

ax.set_xlabel("$\\mathrm{NO_x}$ concentration (ppb)")
ax.set_xscale("log")
ax.set_ylabel("Condensation sink ($\\mathrm{s}^{-1}$)")
ax.set_yscale("log")
ax.legend(loc="lower left")
ax.set_title("$\\mathrm{NO_x}$ / Cond. sink")
ax.set_title("(a)", loc="left")


# O3 and NOx
ax = fig.add_subplot(gs[0,1])

corr = plot_logy_fit_line(ax, data.loc[summer, "O3"], data.loc[summer, "NOx"], "C1")
ax.scatter(data.loc[summer, "O3"], data.loc[summer, "NOx"],
    color="C1", marker=",", s=3, label=f"Jun-Aug ($|\\rho|={{{corr:.2}}}$)")

corr = plot_logy_fit_line(ax, data.loc[winter, "O3"], data.loc[winter, "NOx"], "C0")
ax.scatter(data.loc[winter, "O3"], data.loc[winter, "NOx"],
    color="C0", marker=".", s=9, label=f"Dec-Feb ($|\\rho|={{{corr:.2}}}$)")

ax.set_xlabel("$\\mathrm{O_3}$ concentration (ppb)")
ax.set_ylabel("$\\mathrm{NO_x}$ concentration (ppb)")
ax.set_yscale("log")
ax.legend(loc="lower left")
ax.set_title("$\\mathrm{O_3}$ / $\\mathrm{NO_x}$")
ax.set_title("(b)", loc="left")


# Wind direction and NOx
ax = fig.add_subplot(gs[0,2])

corr = plot_triglog_fit_line(ax, data.loc[summer, "WD"], data.loc[summer, "NOx"], "C1")
ax.scatter(data.loc[summer, "WD"], data.loc[summer, "NOx"],
    color="C1", marker=",", s=3, label=f"Jun-Aug ($|\\rho|={{{corr:.2}}}$)")

corr = plot_triglog_fit_line(ax, data.loc[winter, "WD"], data.loc[winter, "NOx"], "C0")
ax.scatter(data.loc[winter, "WD"], data.loc[winter, "NOx"],
    color="C0", marker=".", s=9, label=f"Dec-Feb ($|\\rho|={{{corr:.2}}}$)")

ax.set_xlabel("Wind direction")
ax.set_xticks([0, 90, 180, 270, 360])
ax.set_xticklabels(["N", "E", "S", "W", "N"])
ax.set_ylabel("$\\mathrm{NO_x}$ concentration (ppb)")
ax.set_yscale("log")
ax.legend(loc="lower left")
ax.set_title("Wind dir. / $\\mathrm{NO_x}$")
ax.set_title("(c)", loc="left")



plt.tight_layout()
plt.savefig("fig_scatters.pdf", transparent=True)
